#include "EcmUsrDriver.h"
#include "platform.h"
#include "Utility.h"
#include "PdoDefine.h"
/*     V
 *     |
 *     |   _______
 *     |  /       \
 *     | /         \
 *___T_|/           \
 *     |             \            /
 *     |              \          /
 *     |               \________/
 *     |
*/
int CalVel(int nAccStep, int nFeedSpeed, int nCostTime)
{
	int nAcc = 1;
	static int nVel=0, nDir=1, ConstTime=0, nAccTime=1;
	if(nDir*nVel >= nFeedSpeed){
		nDir = -1*nDir;
		ConstTime = nCostTime;
	}
	if(ConstTime){
		ConstTime--;
	}else{
		if(nAccTime == nAccStep){
			nVel += nDir*nAcc;
			nAccTime = 1;
		}else{
			nAccTime++;
		}
	}
	return nVel;
}
int SetDriverCmdOpMode(int nDriveCnt, uint8_t u8Mode)
{
	uint8_t u8OpMode;
	int i,nret;
	//	SDO example : use SDO write to set driver mode
	for(i=0;i<nDriveCnt;i++){
		u8OpMode = u8Mode;	// 8 : CSP mode
							// 9 : CSV mode
		// Send write request
		nret = ECM_EcatSdoWrite(i, 0x6060, 0, 1, 7000000, &u8OpMode);
		if(nret <= 0){
			printf("(%d) ECM_EcatSdoWrite error\n", i);
			return -1;
		}
		u8OpMode = 0;
		// Send read request and get response
		nret = ECM_EcatSdoRead(i, 0x6060, 0, 1, 7000000, &u8OpMode);
		if(nret <= 0){
			printf("(%d) ECM_EcatSdoRead error (%d)\n", i, nret);
			return -2;
		}
		if(u8OpMode != u8Mode){
			printf("(%d) OpMode is not correct (%d)\n",i ,u8OpMode);
			return -3;
		}
	}
	return 1;
}
int SetPdoConfTbl(PDO_CONFIG_HEAD *pConfig, uint8_t u8PdoIdx, uint8_t u8TblIdx, uint16_t u16Idx, uint8_t u8SubIdx, uint8_t u8BitSize)
{
	pConfig->Table[u8PdoIdx][u8TblIdx].u16Idx = u16Idx;
	pConfig->Table[u8PdoIdx][u8TblIdx].u8SubIdx = u8SubIdx;
	pConfig->Table[u8PdoIdx][u8TblIdx].u8BitSize = u8BitSize;
	return 1;
}
int SdoShowPdoConfig(int nDriveCnt, uint16_t SmaIdx)
{
	int i,j,k,nret;
	uint8_t u8PdoCnt=0;
	uint16_t u16PdoIdx[4];
	uint8_t u8PdoObjCnt[4];
	uint32_t u32PdoObj[4][16];
	printf("\r\n========== 0x%X ==========\r\n",SmaIdx);
	for(i = 0; i < nDriveCnt; i++){
		nret = ECM_EcatSdoRead(i, SmaIdx, 0, sizeof(u8PdoCnt), 7000000, &u8PdoCnt);
		if(nret <= 0){
			return -1;
		}
		printf("Slave(%d)\r\n",i);
		for(j = 0; j < u8PdoCnt; j++){
			nret = ECM_EcatSdoRead(i, SmaIdx, j+1, sizeof(u16PdoIdx[j]), 7000000, (uint8_t *)&u16PdoIdx[j]);
			if(nret <= 0){
				return -1;
			}
			nret = ECM_EcatSdoRead(i, u16PdoIdx[j], 0, sizeof(u8PdoObjCnt[j]), 7000000, &u8PdoObjCnt[j]);
			if(nret <= 0){
				return -1;
			}
			printf("PdoIdx(0x%X)\r\n",u16PdoIdx[j]);
			for(k=0;k<u8PdoObjCnt[j];k++){
				nret = ECM_EcatSdoRead(i, u16PdoIdx[j], k+1, sizeof(u32PdoObj[j][k]), 7000000, (uint8_t *)&u32PdoObj[j][k]);
				if(nret <= 0){
					return -1;
				}
				printf("\t0x%X\r\n",u32PdoObj[j][k]);
			}
		}
	}
	return 1;
}
int ConfigDrive(int nList, int nDriveCnt, int nJustAssign)
{
	int i,nret;
	PDO_CONFIG_HEAD PDOConfig;
	PDOConfig.SmaIdx = RxPDO_ASSIGN_IDX;
	PDOConfig.PDOCnt = 1;
	PDOConfig.MapIdx[0] = RxPDO_MAP_IDX;

	if(nJustAssign){
		PDOConfig.MapIdx[0] = 0x1600;
		PDOConfig.ObjsCnt[0] = 0;
	}else{
		PDOConfig.MapIdx[0] = RxPDO_MAP_IDX;
		PDOConfig.ObjsCnt[0] = 4;
		//PDOConfig.ObjsCnt[0] = 2;
		SetPdoConfTbl(&PDOConfig, 0, 0, 0x6040, 0, 16);
		SetPdoConfTbl(&PDOConfig, 0, 1, 0x607A, 0, 32);
		SetPdoConfTbl(&PDOConfig, 0, 2, 0x60FF, 0, 32);
		SetPdoConfTbl(&PDOConfig, 0, 3, 0x6060, 0, 8);
	}
	for(i=0;i<nDriveCnt;i++){
		nret=ECM_EcatPdoConfigSet( i, &PDOConfig);
		if(nret <= 0){
			printf("Set RX PDO error\n");
			return -1;
		}
	}
	PDOConfig.SmaIdx = TxPDO_ASSIGN_IDX;
	PDOConfig.PDOCnt = 1;
	if(nJustAssign){
		PDOConfig.MapIdx[0] = 0x1A00;
		PDOConfig.ObjsCnt[0] = 0;
	}else{
		PDOConfig.MapIdx[0] = TxPDO_MAP_IDX;
		PDOConfig.ObjsCnt[0] = 2;
		SetPdoConfTbl(&PDOConfig, 0, 0, 0x6041, 0, 16);
		SetPdoConfTbl(&PDOConfig, 0, 1, 0x6064, 0, 32);
	}
	for(i=0;i<nDriveCnt;i++){
		nret=ECM_EcatPdoConfigSet( i, &PDOConfig);
		if(nret <= 0){
			printf("Set TX PDO error\n");
			return -1;
		}
	}
	nret=ECM_EcatReconfig();
	if( nret < 0 ){
		printf("ECM_ECAT_CONFIG error : %d\n",nret);
		return -1;
	}
	if(nList){
		SdoShowPdoConfig(nDriveCnt, RxPDO_ASSIGN_IDX);
		SdoShowPdoConfig(nDriveCnt, TxPDO_ASSIGN_IDX);
	}
	return 0;
}
/*
 * Check memory space
 *
 * */
int ECM_CheckMEMSpace(int nPdoCntOnce)
{
	int nret;
	uint8_t u8Cnt;
	uint16_t u16PDOSize=0,u16SpiDataSize=0;
	printf("\r\n=====================\r\n");
	printf("  Check : Start\r\n");
	printf("=====================\r\n");
	//
	nret = ECM_GetSpiDataSize(&u16SpiDataSize);
	if(nret <= 0){
		printf("  ERROR : ECM_GetSpiDataSize error\r\n");
		return -1;
	}
	printf("  SPI data size : %d bytes\r\n", u16SpiDataSize);
	//	Check RxFIFO
	ECM_GetRxFIFOCnt(&u8Cnt);
	u16PDOSize = ECM_FifoRxPdoSizeGet();
	if(u16PDOSize == 0){
		printf("  ERROR : ECM_FifoRxPdoSizeGet error\r\n");
		return -2;
	}
	printf("  RxPDO size : %d bytes\r\n", u16PDOSize);
	if(ECM_GetRxFIFOCnt(&u8Cnt)==0){
		printf("  ERROR : ECM_GetRxFIFOCnt error\r\n");
		return -3;
	}
	printf("  RxFIFO MAX. count : %d\r\n", u8Cnt);
	if(u16PDOSize*u8Cnt > RXFIFO_QUEUE_SIZE){
		printf("  ERROR : RxFIFO size explosion(%d > %d)\r\n",u16PDOSize*u8Cnt,RXFIFO_QUEUE_SIZE);
		return -4;
	}
	printf("  RxFIFO queue size : %d bytes\r\n", u16PDOSize*u8Cnt);
	if(u16PDOSize*nPdoCntOnce > u16SpiDataSize){
		printf("  ERROR : RxPDO size bigger than SPI data size(%d > %d)\r\n",u16PDOSize*nPdoCntOnce,u16SpiDataSize);
		return -5;
	}
	//	Check TxFIFO
	ECM_GetTxFIFOCnt(&u8Cnt);
	u16PDOSize = ECM_FifoTxPdoSizeGet();
	if(u16PDOSize == 0){
		printf("  ERROR : ECM_FifoTxPdoSizeGet error\r\n");
		return -2;
	}
	printf("  TxPDO size : %d bytes\r\n", u16PDOSize);
	if(ECM_GetTxFIFOCnt(&u8Cnt)==0){
		printf("  ERROR : ECM_GetTxFIFOCnt error\r\n");
		return -3;
	}
	printf("  TxFIFO MAX. count : %d\r\n", u8Cnt);
	if(u16PDOSize*u8Cnt > TXFIFO_QUEUE_SIZE){
		printf("  ERROR : TxFIFO size explosion(%d > %d)\r\n",u16PDOSize*u8Cnt,TXFIFO_QUEUE_SIZE);
		return -4;
	}
	printf("  TxFIFO queue size : %d bytes\r\n", u16PDOSize*u8Cnt);
	if(u16PDOSize*nPdoCntOnce > u16SpiDataSize){
		printf("  ERROR : TxPDO size bigger than SPI data size(%d > %d)\r\n",u16PDOSize*nPdoCntOnce,u16SpiDataSize);
		return -5;
	}
	printf("\r\n=====================\r\n");
	printf("  Check : PASS\r\n");
	printf("=====================\r\n");
	return 1;
}
